@@ -1,4 +1,3 @@
-import sys
 import warnings
 
 try:
@@ -81,12 +80,13 @@ class SessionMiddleware(object):
     def __init__(self, wrap_app, config=None, environ_key='beaker.session', **kwargs):
         """Initialize the Session Middleware
         
-        The Session middleware will make a lazy session instance available 
-        every request under the ``environ['beaker.cache']`` key by default. The location in
-        environ can be changed by setting ``environ_key``.
+        The Session middleware will make a lazy session instance available
+        every request under the ``environ['beaker.session']`` key by
+        default. The location in environ can be changed by setting
+        ``environ_key``.
         
         ``config``
-            dict  All settings should be prefixed by 'cache.'. This method of
+            dict  All settings should be prefixed by 'session.'. This method of
             passing variables is intended for Paste and other setups that
             accumulate multiple component settings in a single dictionary. If
             config contains *no cache. prefixed args*, then *all* of the config
